"""These are bad uses of _()"""

# pylint: disable=missing-docstring

def _(text):
    return text

def gettext(text):
    return text

def welcome(name):
    _("Hello"+"There")
    _(17)
    _("Hi, {0}".format(name))
    # pylint: disable=translation-of-non-string
    _(name)
    # pylint: enable=translation-of-non-string
    gettext("hi, %s" % name)



"""A TestCase class that calls its parent setUp properly."""

import unittest

# pylint: disable=too-few-public-methods, missing-docstring

class GoodTestCase(unittest.TestCase):
    def setUp(self):
        self.i_am_good = True
        super(GoodTestCase, self).setUp()

    def tearDown(self):
        self.i_am_good = True
        super(GoodTestCase, self).tearDown()

    def test_something(self):
        self.assertEqual(1, 1)



"""These are bad """

# pylint: disable=missing-docstring
# pylint: disable=too-few-public-methods

import unittest

# This is bad

class TestCase(unittest.TestCase):
    def test_one(self):
        pass

class DerivedTestCase(TestCase):
    def test_two(self):
        pass

# This is bad, but the author seems to know better.

class DerivedTestCase2(TestCase):       # pylint: disable=test-inherits-tests
    def test_two(self):
        pass

# No big deal, the base class isn't a test.

class TestHelpers(unittest.TestCase):
    __test__ = False
    def test_one(self):
        pass

class TestsWithHelpers(TestHelpers):
    __test__ = True
    def test_two(self):
        pass

# Bad: this base class is a test.

class TestHelpers2(unittest.TestCase):
    __test__ = True
    def test_one(self):
        pass

class TestsWithHelpers2(TestHelpers2):
    def test_two(self):
        pass

# Mixins are fine.

class TestMixins(object):
    def test_one(self):
        pass

class TestsWithMixins(TestMixins, unittest.TestCase):
    def test_two(self):
        pass

# A base class which is a TestCase, but has no test methods.

class EmptyTestCase(unittest.TestCase):
    def setUp(self):
        super(EmptyTestCase, self).setUp()

class ActualTestCase(EmptyTestCase):
    def test_something(self):
        pass

# Bizzaro __test__ examples to complete branch coverage.

class WhatIsThis(unittest.TestCase):
    def __test__(self):
        return self.fail("I don't know what I'm doing.")


class TooTrickyForTheirOwnGood(unittest.TestCase):
    __test__ = 1 - 1



"""These are bad uses of getattr() and setattr()"""

# pylint: disable=missing-docstring

def do_things(name):
    hello = getattr(name, "hello")
    world = getattr(name, "hello", 17)
    setattr(name, "hello", hello)
    setattr(name, "h" + "ello", world)
    delattr(name, "something")
    delattr(name, "FOO".lower())

    # the message can be disabled:
    hello = getattr(name, "hello")      # pylint: disable=literal-used-as-attribute

    # Account for this case in our code...
    world = getattr(name, 1)



"""Bad uses of range()."""

import six

if six.PY3:
    def xrange(*args_unused):       # pylint: disable=unused-argument, redefined-builtin
        """Just to keep PY3 happy."""

i = 12

range(0, 10)
range(0, i)
range(0, i, 1)
xrange(0, 10)

# pylint: disable=simplifiable-range
range(0, 1000, 1)
# pylint: enable=simplifiable-range
range(0, 10, 1)

# no message when variables are involved
START, STOP, STEP = 0, 10, 1
range(START, 100)
range(START, STOP)
range(0, 10, STEP)

# if it has four arguments, we don't know what's going on...
range(0, 10, 1, "something")
